@@ -8,8 +8,8 @@ from labware_domain_models import LabwareDefinition
 
 try:
     from importlib import metadata
-except ImportError:
-    import importlib_metadata as metadata
+except ImportError:  # pragma: no cover
+    import importlib_metadata as metadata  # type: ignore
 PACKAGE_VERSION = metadata.version("pulse3D")
 
 CURI_BIO_ACCOUNT_UUID = uuid.UUID("73f52be0-368c-42d8-a1fd-660d49ba5604")
@@ -21,17 +21,13 @@ MIN_SUPPORTED_FILE_VERSION = "0.1.1"
 CURRENT_BETA1_HDF5_FILE_FORMAT_VERSION = "0.4.2"
 CURRENT_BETA2_HDF5_FILE_FORMAT_VERSION = "1.0.0"
 FILE_FORMAT_VERSION_METADATA_KEY = "File Format Version"
-FILE_MIGRATION_PATHS = immutabledict(
-    {
-        "0.3.1": "0.4.1",
-        "0.4.1": "0.4.2",
-    }
-)
+FILE_MIGRATION_PATHS = immutabledict({"0.3.1": "0.4.1", "0.4.1": "0.4.2"})
 
 NOT_APPLICABLE_H5_METADATA = uuid.UUID(
     "59d92e00-99d5-4460-9a28-5a1a0fe9aecf"
 )  # Eli (1/19/21): H5 files can't store the concept of `None` in their metadata, so using this value to denote that a particular piece of metadata is not available (i.e. after migrating to a newer file format version)
 
+
 HARDWARE_TEST_RECORDING_UUID = uuid.UUID("a2e76058-08cd-475d-a55d-31d401c3cb34")
 UTC_BEGINNING_DATA_ACQUISTION_UUID = uuid.UUID("98c67f22-013b-421a-831b-0ea55df4651e")
 START_RECORDING_TIME_INDEX_UUID = uuid.UUID("e41422b3-c903-48fd-9856-46ff56a6534c")
@@ -59,9 +55,11 @@ ADC_GAIN_SETTING_UUID = uuid.UUID("a3c3bb32-9b92-4da1-8ed8-6c09f9c816f8")
 ADC_TISSUE_OFFSET_UUID = uuid.UUID("41069860-159f-49f2-a59d-401783c1ecb4")
 ADC_REF_OFFSET_UUID = uuid.UUID("dc10066c-abf2-42b6-9b94-5e52d1ea9bfc")
 PLATE_BARCODE_UUID = uuid.UUID("cf60afef-a9f0-4bc3-89e9-c665c6bb6941")
+STIM_BARCODE_UUID = uuid.UUID("6fa67db1-c8b9-4937-b93f-6fe8bdc7e6d7")
 BACKEND_LOG_UUID = uuid.UUID("87533deb-2495-4430-bce7-12fdfc99158e")
 COMPUTER_NAME_HASH_UUID = uuid.UUID("fefd0675-35c2-45f6-855a-9500ad3f100d")
-BARCODE_IS_FROM_SCANNER_UUID = uuid.UUID("7d026e86-da70-4464-9181-dc0ce2d47bd1")
+PLATE_BARCODE_IS_FROM_SCANNER_UUID = uuid.UUID("7d026e86-da70-4464-9181-dc0ce2d47bd1")
+STIM_BARCODE_IS_FROM_SCANNER_UUID = uuid.UUID("6e5a4b3e-f766-4638-80f7-d95c417c0fc2")
 IS_FILE_ORIGINAL_UNTRIMMED_UUID = uuid.UUID("52231a24-97a3-497a-917c-86c780d9993f")
 TRIMMED_TIME_FROM_ORIGINAL_START_UUID = uuid.UUID("371996e6-5e2d-4183-a5cf-06de7058210a")
 TRIMMED_TIME_FROM_ORIGINAL_END_UUID = uuid.UUID("55f6770d-c369-42ce-a437-5ed89c3cb1f8")
@@ -73,6 +71,12 @@ TOTAL_WORKING_HOURS_UUID = uuid.UUID("f8108718-2fa0-40ce-a51a-8478e5edd4b8")
 TAMPER_FLAG_UUID = uuid.UUID("68d0147f-9a84-4423-9c50-228da16ba895")
 PCB_SERIAL_NUMBER_UUID = uuid.UUID("5103f995-19d2-4880-8a2e-2ce9080cd2f5")
 MAGNETOMETER_CONFIGURATION_UUID = uuid.UUID("921121e9-4191-4536-bedd-03186fa1e117")
+UTC_BEGINNING_STIMULATION_UUID = uuid.UUID("4b310594-ded4-45fd-a1b4-b829aceb416c")
+STIMULATION_PROTOCOL_UUID = uuid.UUID("ede638ce-544e-427a-b1d9-c40784d7c82d")
+IS_CALIBRATION_FILE_UUID = uuid.UUID("9a6f90eb-fe34-423b-bfed-fb441d6d9e5f")
+CHANNEL_FIRMWARE_VERSION_UUID = uuid.UUID("d9694cfe-824c-41f8-915e-91e41ce7af32")
+BOOT_FLAGS_UUID = uuid.UUID("762f6715-ffcd-4e8d-b707-638dd5777841")
+INITIAL_MAGNET_FINDING_PARAMS_UUID = uuid.UUID("da5f2f6d-6874-4e53-be10-90c4bfbd3d45")
 METADATA_UUID_DESCRIPTIONS = immutabledict(
     {
         # General values
@@ -99,12 +103,12 @@ METADATA_UUID_DESCRIPTIONS = immutabledict(
         REF_SAMPLING_PERIOD_UUID: "Reference Sensor Sampling Period (microseconds)",
         TISSUE_SAMPLING_PERIOD_UUID: "Tissue Sensor Sampling Period (microseconds)",
         ADC_GAIN_SETTING_UUID: "ADC Gain Setting",
-        ADC_TISSUE_OFFSET_UUID: "ADC Tissue Sensor Offset",  # may not be needed in Beta 2
-        ADC_REF_OFFSET_UUID: "ADC Reference Sensor Offset",  # may not be needed in Beta 2
+        ADC_TISSUE_OFFSET_UUID: "ADC Tissue Sensor Offset",
+        ADC_REF_OFFSET_UUID: "ADC Reference Sensor Offset",
         PLATE_BARCODE_UUID: "Plate Barcode",
         BACKEND_LOG_UUID: "Backend log file identifier",
         COMPUTER_NAME_HASH_UUID: "SHA512 digest of computer name",
-        BARCODE_IS_FROM_SCANNER_UUID: "Is this barcode obtained from the scanner",
+        PLATE_BARCODE_IS_FROM_SCANNER_UUID: "Is this plate barcode obtained from the scanner",
         IS_FILE_ORIGINAL_UNTRIMMED_UUID: "Is this an original file straight from the instrument and untrimmed",
         TRIMMED_TIME_FROM_ORIGINAL_START_UUID: "Number of centimilliseconds that has been trimmed off the beginning of when the original data started",
         TRIMMED_TIME_FROM_ORIGINAL_END_UUID: "Number of centimilliseconds that has been trimmed off the end of when the original data ended",
@@ -119,6 +123,14 @@ METADATA_UUID_DESCRIPTIONS = immutabledict(
         TAMPER_FLAG_UUID: "Is it suspected the internals of the Mantarray enclosure have been tampered with",
         PCB_SERIAL_NUMBER_UUID: "The serial number of the Mantarray PCB",
         MAGNETOMETER_CONFIGURATION_UUID: "The state (on/off) of the board's magnetometers",
+        UTC_BEGINNING_STIMULATION_UUID: "UTC Timestamp of Beginning of Stimulation",
+        STIMULATION_PROTOCOL_UUID: "The stimulation protocol that was running on this well during recording. Empty string if stimulation was not active",
+        STIM_BARCODE_UUID: "Stim Lid Barcode",
+        STIM_BARCODE_IS_FROM_SCANNER_UUID: "Is this stim lid barcode obtained from the scanner",
+        IS_CALIBRATION_FILE_UUID: "Is this file a calibration (empty plate) recording",
+        CHANNEL_FIRMWARE_VERSION_UUID: "Firmware Version (Channel Controller)",
+        BOOT_FLAGS_UUID: "Hardware/firmware flags present on device bootup",
+        INITIAL_MAGNET_FINDING_PARAMS_UUID: "JSON string of the initial magnet finding params that should be used in Pulse3D",
     }
 )
 
@@ -128,6 +140,7 @@ MICRO_TO_BASE_CONVERSION = int(1e6)
 MICROSECONDS_PER_CENTIMILLISECOND = 10
 TISSUE_SENSOR_READINGS = "tissue_sensor_readings"
 REFERENCE_SENSOR_READINGS = "reference_sensor_readings"
+STIMULATION_READINGS = "stimulation_readings"
 TIME_INDICES = "time_indices"
 TIME_OFFSETS = "time_offsets"
 
@@ -188,10 +201,10 @@ ALL_METRICS = [
     CONTRACTION_VELOCITY_UUID,
     RELAXATION_VELOCITY_UUID,
     IRREGULARITY_INTERVAL_UUID,
-    BASELINE_TO_PEAK_UUID,
-    PEAK_TO_BASELINE_UUID,
     WIDTH_UUID,
     RELAXATION_TIME_UUID,
+    PEAK_TO_BASELINE_UUID,
+    BASELINE_TO_PEAK_UUID,
     CONTRACTION_TIME_UUID,
 ]
 
@@ -205,12 +218,12 @@ BESSEL_LOWPASS_10_UUID = uuid.UUID("7d64cac3-b841-4912-b734-c0cf20a81e7a")
 BESSEL_LOWPASS_30_UUID = uuid.UUID("eee66c75-4dc4-4eb4-8d48-6c608bf28d91")
 BUTTERWORTH_LOWPASS_30_UUID = uuid.UUID("de8d8cef-65bf-4119-ada7-bdecbbaa897a")
 
-# General mangetic field to force conversion factor. Obtained 03/09/2021 by Kevin Grey, Valid as of 11/19/21
+# General mangetic field to force conversion factor. Obtained 03/09/2021 by Kevin Gray, Valid as of 11/19/21
 MILLIMETERS_PER_MILLITESLA = 23.25
 NEWTONS_PER_MILLIMETER = 0.000159
 
 # Beta 1 GMR to magnetic field conversion values. Valid as of 11/19/21
-MILLIVOLTS_PER_MILLITESLA = 1073.6  # Obtained 03/09/2021 by Kevin Grey
+MILLIVOLTS_PER_MILLITESLA = 1073.6  # Obtained 03/09/2021 by Kevin Gray
 MIDSCALE_CODE = 0x800000
 RAW_TO_SIGNED_CONVERSION_VALUE = 2 ** 23  # subtract this value from raw hardware data
 REFERENCE_VOLTAGE = 2.5
@@ -228,7 +241,7 @@ MIN_NUMBER_VALLEYS = 3
 
 
 """
-curibio.sdk constants
+pulse3D constants
 """
 METADATA_EXCEL_SHEET_NAME = "metadata"
 METADATA_RECORDING_ROW_START = 0
@@ -252,14 +265,15 @@ TSP_TO_DEFAULT_FILTER_UUID = {  # Tissue Sampling Period (µs) to default Pipeli
 }
 
 DEFAULT_CELL_WIDTH = 64
+CHART_ALPHA = 60  # for full/snapshots -- num pixels between left figure edge and plot area
+CHART_GAMMA = 150  # for full/snapshots -- num pixels between right figure edge and plot area
+CHART_PIXELS_PER_SECOND = 35  # for full/snapshots -- number of pixels per second
+CHART_MAXIMUM_SNAPSHOT_LENGTH = 10
 CHART_HEIGHT = 300
-CHART_BASE_WIDTH = 120
 CHART_HEIGHT_CELLS = 15
 CHART_FIXED_WIDTH_CELLS = 8
 CHART_FIXED_WIDTH = DEFAULT_CELL_WIDTH * CHART_FIXED_WIDTH_CELLS
 PEAK_VALLEY_COLUMN_START = 100
-CHART_WINDOW_NUM_SECONDS = 10
-CHART_WINDOW_NUM_DATA_POINTS = CHART_WINDOW_NUM_SECONDS / INTERPOLATED_DATA_PERIOD_SECONDS
 SECONDS_PER_CELL = 2.5
 
 CALCULATED_METRIC_DISPLAY_NAMES = {
@@ -275,10 +289,28 @@ CALCULATED_METRIC_DISPLAY_NAMES = {
     WIDTH_UUID: "Twitch Width {} (seconds)",
     RELAXATION_TIME_UUID: "Time From Peak to Relaxation {} (seconds)",
     CONTRACTION_TIME_UUID: "Time From Contraction {} to Peak (seconds)",
-    BASELINE_TO_PEAK_UUID: "Time From Baseline to Peak (seconds)",
-    PEAK_TO_BASELINE_UUID: "Time From Peak to Baseline (seconds)",
+    BASELINE_TO_PEAK_UUID: "Time From Contraction {} to Peak (seconds)",
+    PEAK_TO_BASELINE_UUID: "Time From Peak to Relaxation {} (seconds)",
 }
 
+CALCULATED_METRICS = immutabledict(
+    {
+        "by_width": (WIDTH_UUID, CONTRACTION_TIME_UUID, RELAXATION_TIME_UUID),
+        "scalar": (
+            AMPLITUDE_UUID,
+            AUC_UUID,
+            BASELINE_TO_PEAK_UUID,
+            CONTRACTION_VELOCITY_UUID,
+            FRACTION_MAX_UUID,
+            IRREGULARITY_INTERVAL_UUID,
+            PEAK_TO_BASELINE_UUID,
+            RELAXATION_VELOCITY_UUID,
+            TWITCH_FREQUENCY_UUID,
+            TWITCH_PERIOD_UUID,
+        ),
+    }
+)
+
 COORDS = (10, 25, 50, 75, 90)
 TWITCH_WIDTH_METRIC_DISPLAY_NAMES: Dict[int, str] = immutabledict(
     (coord, f"Twitch Width {coord} (seconds)") for coord in reversed(COORDS)
@@ -328,12 +360,5 @@ EXCEL_OPTICAL_METADATA_CELLS = immutabledict(
 Magnet Finding
 """
 
-WELL_IDX_TO_MODULE_ID = immutabledict(
-    {well_idx: well_idx % 4 * 6 + well_idx // 4 + 1 for well_idx in range(24)}
-)
-MODULE_ID_TO_WELL_IDX = immutabledict(
-    {module_id: well_idx for well_idx, module_id in WELL_IDX_TO_MODULE_ID.items()}
-)
-
 # 10 seconds at sampling rate of 100Hz
 BASELINE_MEAN_NUM_DATA_POINTS = 10 * 100
