@@ -1,36 +1,44 @@
 # -*- coding: utf-8 -*-
 import datetime
 import glob
+import json
 import logging
 import math
 import os
+import copy
 import tempfile
 from typing import Any
 from typing import Optional
+from typing import Union
 import uuid
 import zipfile
 
 import h5py
+from mantarray_magnet_finding.utils import calculate_magnetic_flux_density_from_memsic
 from nptyping import NDArray
 import numpy as np
 from openpyxl import load_workbook
 from openpyxl.worksheet.worksheet import Worksheet
+import pandas as pd
+from scipy import interpolate
 from semver import VersionInfo
 from xlsxwriter.utility import xl_cell_to_rowcol
 
 from .compression_cy import compress_filtered_magnetic_data
 from .constants import *
 from .magnet_finding import find_magnet_positions
+from .magnet_finding import fix_dropped_samples
 from .magnet_finding import format_well_file_data
 from .transforms import apply_empty_plate_calibration
 from .transforms import apply_noise_filtering
 from .transforms import apply_sensitivity_calibration
 from .transforms import calculate_displacement_from_voltage
 from .transforms import calculate_force_from_displacement
-from .transforms import calculate_magnetic_flux_density_from_memsic
 from .transforms import calculate_voltage_from_gmr
 from .transforms import create_filter
 from .transforms import noise_cancellation
+from .utils import truncate
+from .utils import truncate_float
 
 log = logging.getLogger(__name__)
 
@@ -46,7 +54,9 @@ class MantarrayH5FileCreator(h5py.File):
         super().__init__(
             file_name,
             "w",
-            libver="latest",  # Eli (2/9/20) tried to specify this ('earliest', 'v110') to be more backward compatible but it didn't work for unknown reasons (gave error when trying to set swmr_mode=True)
+            # tried to specify this ('earliest', 'v110') to be more backward compatible
+            # but it didn't work for unknown reasons (gave error when trying to set swmr_mode=True)
+            libver="latest",
             userblock_size=512,  # minimum size is 512 bytes
         )
 
@@ -75,17 +85,14 @@ class WellFile:
                 self[UTC_FIRST_TISSUE_DATA_POINT_UUID] = self._extract_datetime(
                     UTC_FIRST_TISSUE_DATA_POINT_UUID
                 )
-                if self.version < VersionInfo.parse(
-                    "1.0.0"
-                ):  # Tanner (12/6/21): Ref data not yet added to these files
+                if self.version < VersionInfo.parse("1.0.0"):  # Ref data not yet added to these files
                     self[UTC_FIRST_REF_DATA_POINT_UUID] = self._extract_datetime(
                         UTC_FIRST_REF_DATA_POINT_UUID
                     )
 
                 # setup noise filter
-                if self.version < VersionInfo.parse(
-                    "1.0.0"
-                ):  # Tanner (12/6/21): should probably add beta 2 file support here and remove this condition
+                if self.version < VersionInfo.parse("1.0.0"):
+                    # should probably add beta 2 file support here and remove this condition
                     self.tissue_sampling_period = (
                         sampling_period if sampling_period else self[TISSUE_SAMPLING_PERIOD_UUID]
                     )
@@ -99,6 +106,14 @@ class WellFile:
                         if self.noise_filter_uuid
                         else None
                     )
+                else:
+                    self.noise_filter_uuid = None
+                    self.filter_coefficients = None
+                    self.tissue_sampling_period = (
+                        sampling_period if sampling_period else self[TISSUE_SAMPLING_PERIOD_UUID]
+                    )
+                    self.is_magnetic_data = False
+
                 is_untrimmed = self.get(IS_FILE_ORIGINAL_UNTRIMMED_UUID, True)
                 time_trimmed = None if is_untrimmed else self.attrs[TRIMMED_TIME_FROM_ORIGINAL_START_UUID]
 
@@ -112,13 +127,11 @@ class WellFile:
                     )
                     self._load_magnetic_data()
                 else:
-                    for reading_type in (
-                        TIME_INDICES,
-                        TIME_OFFSETS,
-                        TISSUE_SENSOR_READINGS,
-                        REFERENCE_SENSOR_READINGS,
-                    ):
-                        self[reading_type] = h5_file[reading_type][:]
+                    self[TIME_INDICES] = h5_file[TIME_INDICES][:]
+                    self[TIME_OFFSETS] = h5_file[TIME_OFFSETS][:]
+                    self[TISSUE_SENSOR_READINGS] = h5_file[TISSUE_SENSOR_READINGS][:]
+                    self[REFERENCE_SENSOR_READINGS] = h5_file[REFERENCE_SENSOR_READINGS][:]
+
         elif file_path.endswith(".xlsx"):
             self._excel_sheet = _get_single_sheet(file_path)
             self.file_name = os.path.basename(file_path)
@@ -129,6 +142,17 @@ class WellFile:
                 "y" in str(_get_excel_metadata_value(self._excel_sheet, TWITCHES_POINT_UP_UUID)).lower()
             )
 
+            self.noise_filter_uuid = (
+                TSP_TO_DEFAULT_FILTER_UUID[self.tissue_sampling_period] if self.is_magnetic_data else None
+            )
+            self.filter_coefficients = (
+                create_filter(self.noise_filter_uuid, self.tissue_sampling_period)
+                if self.noise_filter_uuid
+                else None
+            )
+
+            self._load_magnetic_data()
+
     def _load_magnetic_data(self):
         adj_raw_tissue_reading = self[TISSUE_SENSOR_READINGS].copy()
         f = MICROSECONDS_PER_CENTIMILLISECOND if self.is_magnetic_data else MICRO_TO_BASE_CONVERSION
@@ -176,19 +200,22 @@ class WellFile:
             self.compressed_voltage
         )
         self.compressed_force: NDArray[(2, Any), np.float32] = calculate_force_from_displacement(
-            self.compressed_displacement
+            self.compressed_displacement, in_mm=False
         )
 
         self.voltage: NDArray[(2, Any), np.float32] = calculate_voltage_from_gmr(
             self.noise_filtered_magnetic_data
         )
+
         self.displacement: NDArray[(2, Any), np.float64] = calculate_displacement_from_voltage(self.voltage)
-        self.force: NDArray[(2, Any), np.float64] = calculate_force_from_displacement(self.displacement)
+        self.force: NDArray[(2, Any), np.float64] = calculate_force_from_displacement(
+            self.displacement, in_mm=False
+        )
 
     def get(self, key, default):
         try:
             return self[key]
-        except:
+        except Exception:
             return default
 
     def __contains__(self, key):
@@ -282,11 +309,20 @@ class WellFile:
 
 
 class PlateRecording:
-    def __init__(self, path, use_mean_of_baseline=True):
+    def __init__(
+        self,
+        path,
+        use_mean_of_baseline=True,
+        force_df: pd.DataFrame = None,
+        start_time: Union[float, int] = 0,
+        end_time: Union[float, int] = np.inf,
+    ):
         self.path = path
         self.wells = []
         self._iter = 0
         self.is_optical_recording = False
+        self.start_time = start_time * MICRO_TO_BASE_CONVERSION
+        self.end_time = end_time * MICRO_TO_BASE_CONVERSION
 
         if self.path.endswith(".zip"):
             with tempfile.TemporaryDirectory() as tmpdir:
@@ -302,9 +338,14 @@ class PlateRecording:
             self.wells, calibration_recordings = load_files(self.path)
 
         if self.wells:  # might not be any well files in the path
-            # Tanner (12/3/21): currently file versions 1.0.0 and above must have all their data processed together
+            # currently file versions 1.0.0 and above must have all their data processed together
             if not self.is_optical_recording and self.wells[0].version >= VersionInfo.parse("1.0.0"):
-                self._process_plate_data(calibration_recordings, use_mean_of_baseline=use_mean_of_baseline)
+                if force_df is None:
+                    self._process_plate_data(
+                        calibration_recordings, use_mean_of_baseline=use_mean_of_baseline
+                    )
+                else:
+                    self._load_dataframe(force_df)
 
     def _process_plate_data(self, calibration_recordings, use_mean_of_baseline):
         if not all(isinstance(well_file, WellFile) for well_file in self.wells) or len(self.wells) != 24:
@@ -316,9 +357,14 @@ class PlateRecording:
         ):
             raise NotImplementedError("All 24 wells must have a calibration file present")
 
+        initial_magnet_finding_params = json.loads(
+            self.wells[0].get(INITIAL_MAGNET_FINDING_PARAMS_UUID, r"{}")
+        )
+
         # load data
         plate_data_array = format_well_file_data(self.wells)
-        plate_data_array_mt = calculate_magnetic_flux_density_from_memsic(plate_data_array)
+        fixed_plate_data_array = fix_dropped_samples(plate_data_array)
+        plate_data_array_mt = calculate_magnetic_flux_density_from_memsic(fixed_plate_data_array)
         baseline_data = format_well_file_data(calibration_recordings)
         baseline_data_mt = calculate_magnetic_flux_density_from_memsic(baseline_data)
 
@@ -328,26 +374,156 @@ class PlateRecording:
                 baseline_data_mt[:, :, :, -BASELINE_MEAN_NUM_DATA_POINTS:], axis=3
             ).reshape((24, 3, 3, 1))
         else:
-            # extend baseline data (if necessary) so that it has at least as many samples as the recording
+            # extend baseline data (if necessary) so that it has at least as many samples as the tissue data
             num_samples_in_recording = plate_data_array_mt.shape[-1]
             num_samples_in_baseline = baseline_data_mt.shape[-1]
             num_times_to_duplicate = math.ceil(num_samples_in_recording / num_samples_in_baseline)
-            baseline_data_mt = np.tile(baseline_data_mt, num_times_to_duplicate)[:, :, :, :num_samples_in_recording]
+            # truncate so baseline data has exactly as many samples as the tissue data
+            baseline_data_mt = np.tile(baseline_data_mt, num_times_to_duplicate)[
+                :, :, :, :num_samples_in_recording
+            ]
 
         # pass data into magnet finding alg
-        estimated_magnet_positions = find_magnet_positions(plate_data_array_mt, baseline_data_mt)
+        log.info("Estimating magnet positions")
+        estimated_magnet_positions = find_magnet_positions(
+            plate_data_array_mt, baseline_data_mt, initial_magnet_finding_params
+        )
+
+        flip_data = self.wells[0].version >= VersionInfo.parse("1.1.0")
+
+        # create displacement and force arrays for each WellFile
+        log.info("Create diplacement and force data for each well")
+        for well_idx in range(24):
+            well_file = self.wells[well_idx]
+            x = estimated_magnet_positions["X"][:, well_idx]
+            if flip_data:
+                x *= -1
+            # have time indices start at 0
+            adjusted_time_indices = well_file[TIME_INDICES] - well_file[TIME_INDICES][0]
+            start_idx, end_idx = truncate(
+                source_series=adjusted_time_indices,
+                lower_bound=self.start_time,
+                upper_bound=self.end_time,
+            )
+
+            well_file.displacement = np.array(
+                [adjusted_time_indices[start_idx : end_idx + 1], x[start_idx : end_idx + 1]]
+            )
 
-        # create displace and force arrays for each WellFile
-        for module_id in range(1, 25):
-            well_file = self.wells[MODULE_ID_TO_WELL_IDX[module_id]]
-            x = estimated_magnet_positions["X"][:, module_id - 1]
-            well_file.displacement = np.array([well_file[TIME_INDICES], x])
             well_file.force = calculate_force_from_displacement(well_file.displacement)
 
+    def _load_dataframe(self, df: pd.DataFrame) -> None:
+        """Add time and force data to well files in PR.
+
+        Args:
+            df: pd.Dataframe existing time force data to be added
+        """
+        time = df["Time (s)"].values
+        for well in self.wells:
+            well_name = well[WELL_NAME_UUID]
+            well_data = np.vstack((time, df[f"{well_name}__raw"])).astype(np.float64)
+            well.force = well_data
+
+    def write_time_force_csv(self, output_dir: str):
+        # get recording name
+        recording_name = os.path.splitext(os.path.basename(self.path))[0]
+        output_path = os.path.join(output_dir, f"{recording_name}.csv")
+
+        # set indexes to time points
+        truncated_time_sec = [
+            truncate_float(ms / MICRO_TO_BASE_CONVERSION, 2) for ms in self.wells[0].force[0]
+        ]
+
+        force_data = dict({"Time (s)": pd.Series(truncated_time_sec)})
+
+        for well in self.wells:
+            well_name = well.get(WELL_NAME_UUID, None)
+            force_data[well_name] = pd.Series(well.force[1])
+
+        time_force_df = pd.DataFrame(force_data)
+        time_force_df.to_csv(output_path, index=False)
+
+        return time_force_df, output_path
+
+    def to_dataframe(self, from_dataframe: bool = False) -> pd.DataFrame:
+        """
+        Creates DataFrame from PlateRecording with all the data interpolated, normalized, and scaled.
+        The returned dataframe contains one column for time in ms and one column for each well.
+
+        The dataframe returned by this method can be used in calls to peak_detector by selecting the
+        'time' column and the well column that peak detection should be run on after transposing the
+        data, e.g.
+
+        >>> df = to_datafram()
+        >>> peak_detector(df[['time', 'A1']].transpose().values)
+
+        The dataframe needs to be transposed before converting to NDArray because peak_detector
+        wants an ndarray of shape (2,N) while dataframe[['time', 'A1']] is in shape (N,2)
+        """
+        data = {}
+
+        # get first valid well and set interpolation period
+        first_well = [pw for pw in self.wells if pw][0]
+
+        if self.is_optical_recording:
+            interp_period = first_well[INTERPOLATED_VALUE_UUID]
+        else:
+            interp_period = INTERPOLATED_DATA_PERIOD_US
+
+        if from_dataframe:
+            time_steps = copy.deepcopy(first_well.force[0])
+        else:
+            max_time = max([w.force[0][-1] for w in self.wells if w])
+            time_steps = np.arange(interp_period, max_time + interp_period, interp_period)
+
+        data["Time (s)"] = pd.Series(time_steps)
+
+        for i, w in enumerate(self.wells):
+
+            if not from_dataframe:
+                start_idx, end_idx = truncate(
+                    source_series=time_steps,
+                    lower_bound=w.force[0][0],
+                    upper_bound=w.force[0][-1],
+                )
+
+                interp_fn = interpolate.interp1d(w.force[0, :], w.force[1, :])
+                interp_force = interp_fn(time_steps[start_idx : end_idx + 1])
+            else:
+                interp_force = copy.deepcopy(w.force[1, 1:])
+
+            data[f"{w.get(WELL_NAME_UUID, str(i))}__raw"] = pd.Series(w.force[1, :])
+
+            min_value = min(interp_force)
+            interp_force -= min_value
+            interp_force *= MICRO_TO_BASE_CONVERSION
+
+            data[w.get(WELL_NAME_UUID, str(i))] = pd.Series(interp_force)
+
+        df = pd.DataFrame(data)
+        df = df.dropna()
+
+        return df
+
+    @staticmethod
+    def from_dataframe(path, df: pd.DataFrame):
+        # multi zip files
+        # only allowed for one recording at a time assuming a user would only ever pass a dataframe to one recording
+        log.info(f"Loading recording from file {os.path.basename(path)}")
+        yield PlateRecording(path, force_df=df)
+
     @staticmethod
     def from_directory(path):
         # multi zip files
         for zf in glob.glob(os.path.join(path, "*.zip"), recursive=True):
+            zip_file = zipfile.ZipFile(zf)
+            # check if this is a zip of optical files
+            if ".xlsx" in zip_file.namelist()[0]:
+                with tempfile.TemporaryDirectory() as tmpdir:
+                    zip_file.extractall(path=tmpdir)
+                    for optical_file in os.scandir(tmpdir):
+                        yield PlateRecording(optical_file.path)
+                return
             log.info(f"Loading recording from file {zf}")
             yield PlateRecording(zf)
 
@@ -357,8 +533,9 @@ class PlateRecording:
             yield PlateRecording(of)
 
         # directory of .h5 files
-        if glob.glob(os.path.join(path, "**", "*.h5"), recursive=True):
-            yield PlateRecording(path)
+        for dir in glob.glob(os.path.join(path, "*"), recursive=True):
+            if glob.glob(os.path.join(dir, "*.h5"), recursive=True):
+                yield PlateRecording(dir)
 
     def __iter__(self):
         self._iter = 0
@@ -441,8 +618,6 @@ def _get_excel_metadata_value(sheet: Worksheet, metadata_uuid: uuid.UUID) -> Opt
         )
     row, col = xl_cell_to_rowcol(cell_name)
     result = _get_cell_value(sheet, row, col)
-    if result is None and metadata_uuid != INTERPOLATION_VALUE_UUID:
-        raise MetadataNotFoundError(f"Metadata entry not found for {metadata_description}")
     return result
 
 
@@ -451,25 +626,24 @@ def _load_optical_file_attrs(sheet: Worksheet):
 
     value = _get_excel_metadata_value(sheet, TISSUE_SAMPLING_PERIOD_UUID)
     if value is None:
-        raise NotImplementedError(
-            "Tissue Sampling Period should never be None here. A MetadataNotFoundError should have been raised by get_excel_metadata_value"
-        )
+        raise NotImplementedError("Tissue Sampling Period should not be None here")
     sampling_period = int(round(1 / float(value), 6) * MICRO_TO_BASE_CONVERSION)
 
-    interpolation_value = _get_excel_metadata_value(sheet, INTERPOLATION_VALUE_UUID)
-    if interpolation_value is None:
-        interpolation_value = sampling_period
-    else:
-        interpolation_value = float(interpolation_value) * MICRO_TO_BASE_CONVERSION
+    interpolation_value_str = _get_excel_metadata_value(sheet, INTERPOLATION_VALUE_UUID)
+    interpolation_value = (
+        float(sampling_period)
+        if interpolation_value_str is None
+        else float(interpolation_value_str) * MICRO_TO_BASE_CONVERSION
+    )
 
     begin_recording = _get_excel_metadata_value(sheet, UTC_BEGINNING_RECORDING_UUID)
-    begin_recording = datetime.datetime.strptime(begin_recording, "%Y-%m-%d %H:%M:%S")
+    begin_recording = datetime.datetime.strptime(begin_recording, "%Y-%m-%d %H:%M:%S")  # type: ignore
 
     twenty_four_well = LabwareDefinition(row_count=4, column_count=6)
     well_name = _get_excel_metadata_value(sheet, WELL_NAME_UUID)
 
     attrs = {
-        FILE_FORMAT_VERSION_METADATA_KEY: "0.1.1",
+        FILE_FORMAT_VERSION_METADATA_KEY: "N/A",
         TISSUE_SENSOR_READINGS: raw_tissue_reading,
         REFERENCE_SENSOR_READINGS: np.zeros(raw_tissue_reading.shape),
         str(INTERPOLATION_VALUE_UUID): interpolation_value,
